@@ -6,17 +6,18 @@ import traceback
 
 import numpy as np
 
+from qtpy import QtCore, QtGui, QtWidgets, compat
+from qtpy.QtCore import Qt
+
 from glue.external.six.moves import range as xrange
-from glue.external.qt import QtGui
-from glue.external.qt.QtCore import Qt, Signal
 from glue.core.aggregate import Aggregate
 from glue.core.exceptions import IncompatibleAttribute
 from glue.core import Subset
 from glue.core.callback_property import add_callback, ignore_callback
-from glue.config import fit_plugin
-from glue.viewers.common.qt.toolbar import GlueToolbar
+from glue.config import fit_plugin, viewer_tool
+from glue.viewers.common.qt.mpl_toolbar import MatplotlibViewerToolbar
 from glue.core.qt.mime import LAYERS_MIME_TYPE
-from glue.viewers.common.qt.mouse_mode import SpectrumExtractorMode
+from glue.viewers.common.qt.mouse_mode import RoiMode
 from glue.utils.qt import load_ui
 from glue.core.qt.simpleforms import build_form_item
 from glue.utils.qt.widget_properties import CurrentComboProperty
@@ -26,7 +27,7 @@ from glue.utils import nonpartial, Pointer
 from glue.utils.qt import Worker, messagebox_on_error
 from glue.core import roi as core_roi
 from glue.core.subset import RoiSubsetState
-
+from glue.core.qt import roi as qt_roi
 from .profile_viewer import ProfileViewer
 
 
@@ -263,7 +264,7 @@ class NavContext(SpectrumContext):
         pass
 
     def _setup_widget(self):
-        self.widget = QtGui.QTextEdit()
+        self.widget = QtWidgets.QTextEdit()
         self.widget.setHtml("To <b> slide </b> through the cube, "
                             "drag the handle or double-click<br><br><br>"
                             "To make a <b> new profile </b>, "
@@ -283,21 +284,21 @@ class CollapseContext(SpectrumContext):
         self.grip = self.main.profile.new_range_grip()
 
     def _setup_widget(self):
-        w = QtGui.QWidget()
-        l = QtGui.QFormLayout()
+        w = QtWidgets.QWidget()
+        l = QtWidgets.QFormLayout()
         w.setLayout(l)
 
-        combo = QtGui.QComboBox()
+        combo = QtWidgets.QComboBox()
         combo.addItem("Mean", userData=Aggregate.mean)
         combo.addItem("Median", userData=Aggregate.median)
         combo.addItem("Max", userData=Aggregate.max)
         combo.addItem("Centroid", userData=Aggregate.mom1)
         combo.addItem("Linewidth", userData=Aggregate.mom2)
 
-        run = QtGui.QPushButton("Collapse")
-        save = QtGui.QPushButton("Save as FITS file")
+        run = QtWidgets.QPushButton("Collapse")
+        save = QtWidgets.QPushButton("Save as FITS file")
 
-        buttons = QtGui.QHBoxLayout()
+        buttons = QtWidgets.QHBoxLayout()
         buttons.addWidget(run)
         buttons.addWidget(save)
 
@@ -342,7 +343,7 @@ class CollapseContext(SpectrumContext):
     @messagebox_on_error("Failed to export projection")
     def _choose_save(self):
 
-        out, _ = QtGui.QFileDialog.getSaveFileName(filter='FITS Files (*.fits)')
+        out, _ = compat.getsavefilename(filters='FITS Files (*.fits)')
         if out is None:
             return
 
@@ -388,7 +389,7 @@ class CollapseContext(SpectrumContext):
         fits.writeto(pth, self._agg, header, clobber=True)
 
 
-class ConstraintsWidget(QtGui.QWidget):
+class ConstraintsWidget(QtWidgets.QWidget):
 
     """
     A widget to display and tweak the constraints of a :class:`~glue.core.fitters.BaseFitter1D`
@@ -401,56 +402,56 @@ class ConstraintsWidget(QtGui.QWidget):
         constraints : dict
             The `contstraints` property of a :class:`~glue.core.fitters.BaseFitter1D`
             object
-        parent : QtGui.QWidget (optional)
+        parent : QtWidgets.QWidget (optional)
             The parent of this widget
         """
         super(ConstraintsWidget, self).__init__(parent)
         self.constraints = constraints
 
-        self.layout = QtGui.QGridLayout()
+        self.layout = QtWidgets.QGridLayout()
         self.layout.setContentsMargins(2, 2, 2, 2)
         self.layout.setSpacing(4)
 
         self.setLayout(self.layout)
 
-        self.layout.addWidget(QtGui.QLabel("Estimate"), 0, 1)
-        self.layout.addWidget(QtGui.QLabel("Fixed"), 0, 2)
-        self.layout.addWidget(QtGui.QLabel("Bounded"), 0, 3)
-        self.layout.addWidget(QtGui.QLabel("Lower Bound"), 0, 4)
-        self.layout.addWidget(QtGui.QLabel("Upper Bound"), 0, 5)
+        self.layout.addWidget(QtWidgets.QLabel("Estimate"), 0, 1)
+        self.layout.addWidget(QtWidgets.QLabel("Fixed"), 0, 2)
+        self.layout.addWidget(QtWidgets.QLabel("Bounded"), 0, 3)
+        self.layout.addWidget(QtWidgets.QLabel("Lower Bound"), 0, 4)
+        self.layout.addWidget(QtWidgets.QLabel("Upper Bound"), 0, 5)
 
         self._widgets = {}
         names = sorted(list(self.constraints.keys()))
 
         for k in names:
             row = []
-            w = QtGui.QLabel(k)
+            w = QtWidgets.QLabel(k)
             row.append(w)
 
             v = QtGui.QDoubleValidator()
-            e = QtGui.QLineEdit()
+            e = QtWidgets.QLineEdit()
             e.setValidator(v)
             e.setText(str(constraints[k]['value'] or ''))
             row.append(e)
 
-            w = QtGui.QCheckBox()
+            w = QtWidgets.QCheckBox()
             w.setChecked(constraints[k]['fixed'])
             fix = w
             row.append(w)
 
-            w = QtGui.QCheckBox()
+            w = QtWidgets.QCheckBox()
             limits = constraints[k]['limits']
             w.setChecked(limits is not None)
             bound = w
             row.append(w)
 
-            e = QtGui.QLineEdit()
+            e = QtWidgets.QLineEdit()
             e.setValidator(v)
             if limits is not None:
                 e.setText(str(limits[0]))
             row.append(e)
 
-            e = QtGui.QLineEdit()
+            e = QtWidgets.QLineEdit()
             e.setValidator(v)
             if limits is not None:
                 e.setText(str(limits[1]))
@@ -493,7 +494,7 @@ class ConstraintsWidget(QtGui.QWidget):
             fitter.set_constraint(name, **s)
 
 
-class FitSettingsWidget(QtGui.QDialog):
+class FitSettingsWidget(QtWidgets.QDialog):
 
     def __init__(self, fitter, parent=None):
         super(FitSettingsWidget, self).__init__(parent)
@@ -506,7 +507,7 @@ class FitSettingsWidget(QtGui.QDialog):
     def _build_form(self):
         fitter = self.fitter
 
-        l = QtGui.QFormLayout()
+        l = QtWidgets.QFormLayout()
         options = fitter.options
         self.widgets = {}
         self.forms = {}
@@ -524,8 +525,8 @@ class FitSettingsWidget(QtGui.QDialog):
         else:
             self.constraints = None
 
-        self.okcancel = QtGui.QDialogButtonBox(QtGui.QDialogButtonBox.Ok |
-                                               QtGui.QDialogButtonBox.Cancel)
+        self.okcancel = QtWidgets.QDialogButtonBox(QtWidgets.QDialogButtonBox.Ok |
+                                               QtWidgets.QDialogButtonBox.Cancel)
         l.addRow(self.okcancel)
         self.setLayout(l)
 
@@ -625,7 +626,7 @@ class FitContext(SpectrumContext):
         self.canvas.draw()
 
 
-class SpectrumMainWindow(QtGui.QMainWindow):
+class SpectrumMainWindow(QtWidgets.QMainWindow):
 
     """
     The main window that the spectrum viewer is embedded in.
@@ -633,8 +634,8 @@ class SpectrumMainWindow(QtGui.QMainWindow):
     Defines two signals to trigger when a subset is dropped into the window,
     and when the window is closed.
     """
-    subset_dropped = Signal(object)
-    window_closed = Signal()
+    subset_dropped = QtCore.Signal(object)
+    window_closed = QtCore.Signal()
 
     def __init__(self, parent=None):
         super(SpectrumMainWindow, self).__init__(parent=parent)
@@ -656,6 +657,74 @@ class SpectrumMainWindow(QtGui.QMainWindow):
             self.subset_dropped.emit(layer)
 
 
+@viewer_tool
+class SpectrumExtractorMode(RoiMode):
+
+    """
+    Lets the user select a region in an image and, when connected to a
+    SpectrumExtractorTool, uses this to display spectra extracted from that
+    position
+    """
+    persistent = True
+
+    icon = 'glue_spectrum'
+    tool_id = 'spectrum'
+    action_text = 'Spectrum'
+    tool_tip = 'Extract a spectrum from the selection'
+    shortcut = 'S'
+
+    def __init__(self, viewer, **kwargs):
+        super(SpectrumExtractorMode, self).__init__(viewer, **kwargs)
+        self._roi_tool = qt_roi.QtRectangularROI(self._axes)  # default
+        self._tool = SpectrumTool(self.viewer, self)
+        self._release_callback = self._tool._update_profile
+        self._move_callback = self._tool._move_profile
+        self._roi_callback = None
+
+    def menu_actions(self):
+
+        result = []
+
+        a = QtWidgets.QAction('Rectangle', None)
+        a.triggered.connect(nonpartial(self.set_roi_tool, 'Rectangle'))
+        result.append(a)
+
+        a = QtWidgets.QAction('Circle', None)
+        a.triggered.connect(nonpartial(self.set_roi_tool, 'Circle'))
+        result.append(a)
+
+        a = QtWidgets.QAction('Polygon', None)
+        a.triggered.connect(nonpartial(self.set_roi_tool, 'Polygon'))
+        result.append(a)
+
+        for r in result:
+            if self._move_callback is not None:
+                r.triggered.connect(nonpartial(self._move_callback, self))
+
+        return result
+
+    def set_roi_tool(self, mode):
+        if mode is 'Rectangle':
+            self._roi_tool = qt_roi.QtRectangularROI(self._axes)
+
+        if mode is 'Circle':
+            self._roi_tool = qt_roi.QtCircularROI(self._axes)
+
+        if mode is 'Polygon':
+            self._roi_tool = qt_roi.QtPolygonalROI(self._axes)
+
+        self._roi_tool.plot_opts.update(edgecolor='#c51b7d',
+                                        facecolor=None,
+                                        edgewidth=3,
+                                        alpha=1.0)
+
+    def close(self):
+        self._tool.close()
+        return super(SpectrumExtractorMode, self).close()
+
+# TODO: refactor this so that we don't have a separate tool and mode
+
+
 class SpectrumTool(object):
 
     """
@@ -669,7 +738,7 @@ class SpectrumTool(object):
     *collapse context* lets the users collapse a section of a cube to a 2D image
     """
 
-    def __init__(self, image_widget):
+    def __init__(self, image_widget, mouse_mode):
         self._relim_requested = True
 
         self.image_widget = image_widget
@@ -679,7 +748,7 @@ class SpectrumTool(object):
         self.profile = ProfileViewer(self.canvas.fig)
         self.axes = self.profile.axes
 
-        self.mouse_mode = self._setup_mouse_mode()
+        self.mouse_mode = mouse_mode
         self._setup_toolbar()
 
         self._setup_ctxbar()
@@ -712,8 +781,8 @@ class SpectrumTool(object):
         self.widget = SpectrumMainWindow()
         self.widget.window_closed.connect(self.reset)
 
-        w = QtGui.QWidget()
-        l = QtGui.QHBoxLayout()
+        w = QtWidgets.QWidget()
+        l = QtWidgets.QHBoxLayout()
         l.setSpacing(2)
         l.setContentsMargins(2, 2, 2, 2)
         w.setLayout(l)
@@ -725,13 +794,17 @@ class SpectrumTool(object):
 
         self.widget.setCentralWidget(w)
 
+        # TODO: fix hacks
+        w.canvas = self.canvas
+        self.widget.central_widget = w
+
     def _setup_ctxbar(self):
         l = self.widget.centralWidget().layout()
         self._contexts = [NavContext(self),
                           FitContext(self),
                           CollapseContext(self)]
 
-        tabs = QtGui.QTabWidget()
+        tabs = QtWidgets.QTabWidget()
         tabs.addTab(self._contexts[0].widget, 'Navigate')
         tabs.addTab(self._contexts[1].widget, 'Fit')
         tabs.addTab(self._contexts[2].widget, 'Collapse')
@@ -756,21 +829,15 @@ class SpectrumTool(object):
 
         self.widget.subset_dropped.connect(self._extract_subset_profile)
 
-    def _setup_mouse_mode(self):
-        # This will be added to the ImageWidget's toolbar
-        mode = SpectrumExtractorMode(self.image_widget.client.axes,
-                                     release_callback=self._update_profile,
-                                     move_callback=self._move_profile)
-        return mode
-
     def _setup_toolbar(self):
-        tb = GlueToolbar(self.canvas, self.widget)
+
+        tb = MatplotlibViewerToolbar(self.widget)
 
         # disable ProfileViewer mouse processing during mouse modes
-        tb.mode_activated.connect(self.profile.disconnect)
-        tb.mode_deactivated.connect(self.profile.connect)
+        tb.tool_activated.connect(self.profile.disconnect)
+        tb.tool_deactivated.connect(self.profile.connect)
 
-        self._menu_toggle_action = QtGui.QAction("Options", tb)
+        self._menu_toggle_action = QtWidgets.QAction("Options", tb)
         self._menu_toggle_action.setCheckable(True)
         self._menu_toggle_action.toggled.connect(self._toggle_menu)
 
